#include "extension.h"
#include "CDetour/detours.h"

CalcPlayerScore g_Interface;
SMEXT_LINK(&g_Interface);

CDetour *CalcPlayerScoreDetour = NULL;
CDetour *UTIL_PlayerByIndexDetour = NULL;

ICvar *icvar = NULL;
IForward *g_CalcPlayerScore = NULL;
IGameConfig *g_pGameConf = NULL;
IServerGameEnts *gameents = NULL;

int g_CurrentPlayer;

ConVar g_Version("cps_version", "1.1.0b", FCVAR_PLUGIN|FCVAR_SPONLY|FCVAR_REPLICATED|FCVAR_NOTIFY, "CalcPlayerScore Version");

DETOUR_DECL_STATIC1(CalcPlayerScore, int, void *, roundStats)
{
	int score = DETOUR_STATIC_CALL(CalcPlayerScore)(roundStats);
	
	if (!g_CurrentPlayer)
	{
		return score;
	}

	cell_t result;
	g_CalcPlayerScore->PushCell(g_CurrentPlayer);
	g_CalcPlayerScore->PushCell(score);
	g_CalcPlayerScore->Execute(&result);

	g_CurrentPlayer = 0;

	return result;
}

DETOUR_DECL_STATIC1(UTIL_PlayerByIndex, edict_t *, int, client)
{
	g_CurrentPlayer = client;

	return DETOUR_STATIC_CALL(UTIL_PlayerByIndex)(client);
}

void SetupDetours()
{
	CalcPlayerScoreDetour = DETOUR_CREATE_STATIC(CalcPlayerScore, "CalcPlayerScore");
	if (CalcPlayerScoreDetour != NULL)
	{
		CalcPlayerScoreDetour->EnableDetour();
	}

	UTIL_PlayerByIndexDetour = DETOUR_CREATE_STATIC(UTIL_PlayerByIndex, "PlayerByIndex");	
	if (UTIL_PlayerByIndexDetour != NULL)
	{
		UTIL_PlayerByIndexDetour->EnableDetour();
	}
}

void RemoveDetours()
{
	if (CalcPlayerScoreDetour != NULL)
	{
		CalcPlayerScoreDetour->Destroy();
	}

	if (UTIL_PlayerByIndexDetour != NULL)
	{
		UTIL_PlayerByIndexDetour->Destroy();
	}
}


bool CalcPlayerScore::SDK_OnLoad(char *error, size_t maxlength, bool late)
{
	char conf_error[255] = "";
	if (!gameconfs->LoadGameConfigFile("calcplayerscore", &g_pGameConf, conf_error, sizeof(conf_error)))
	{
		if (strlen(conf_error) > 0)
		{
			snprintf(error, maxlength, "Could not read calcplayerscore.txt: %s", conf_error);
		}
		return false;
	}

	CDetourManager::Init(g_pSM->GetScriptingEngine(), g_pGameConf);
	SetupDetours();

	return true;
}

void CalcPlayerScore::SDK_OnUnload()
{
	forwards->ReleaseForward(g_CalcPlayerScore);
	RemoveDetours();
}

void CalcPlayerScore::SDK_OnAllLoaded()
{
	g_CalcPlayerScore = forwards->CreateForward("OnCalcPlayerScore", ET_Event, 2, NULL, Param_Cell, Param_Cell);
}

class BaseAccessor : public IConCommandBaseAccessor
{
public:
	bool RegisterConCommandBase(ConCommandBase *pCommandBase)
	{
		/* Always call META_REGCVAR instead of going through the engine. */
		return META_REGCVAR(pCommandBase);
	}
} s_BaseAccessor;

bool CalcPlayerScore::SDK_OnMetamodLoad(SourceMM::ISmmAPI *ismm, char *error, size_t maxlen, bool late)
{
	GET_V_IFACE_CURRENT(GetEngineFactory, icvar, ICvar, CVAR_INTERFACE_VERSION);

	g_pCVar = icvar;
	ConVar_Register(0, &s_BaseAccessor);

	return true;
}
